==> xlr-status-event-module-22.3.0-902.800.jar: web/include/external-deployments/ducks/external-deployments.saga.ts <==
import { all, call, CallEffect, put, select, take, takeEvery } from 'redux-saga/effects';
import angular, { IHttpResponse } from 'angular';
import { AxiosError } from 'axios';
import { PayloadAction } from '@reduxjs/toolkit';
import { EventChannel } from 'redux-saga';
import { folderExternalDeployments, FolderExternalDeploymentsState } from './external-deployments.reducer';
import {
    DeleteStatusEvent,
    EndpointExternalDeploymentEvent,
    ExternalDeploymentFilters,
    WebhookEndpoint,
    ExternalDeployment,
    SaveWebhookSourceFiltersRequest,
    Server,
    SetupLiveUpdateRequest,
    UpdateStatusEvent,
    WebhookSource,
    WebhookSourceAutoConfigDetails,
    WebhookSourceFilter,
} from '../external-deployment.types';
import { CustomConfiguration } from '../../../../../../../../../core/xlr-ui/app/types';
import { mapOrderByValue, registerEventSource, subscribeToSse } from '../helper/utils';
import { DEFAULT_MAX_AGE, EVENT_TYPE, MAX_AGE_NO_CACHE, SSE_REGISTRATION_URL } from '../constants';
import { getApplicationStatusChannelSelector, getFolderExternalDeployments, getWebhookSourcesSelector } from './external-deployments.selectors';
import ToastrFactory from '../../../../../../../../../core/xlr-ui/app/js/util/toastrFactory';
import { httpGET, httpPOST, httpPUT, httpDELETE } from '../../../../../../../../../core/xlr-ui/app/features/common/services/http';
import { withFlagChangingState } from '../../../../../../../../../core/xlr-ui/app/react/utils/saga-utils';
import getAngularService from '../../../../../../../../../core/xlr-ui/app/features/common/services/angular-accessor';
import { ClientSettings, FilterQueryParams } from '../../../../../../../../../core/xlr-ui/app/types/index';

const {
    createApplicationStatus,
    createServer,
    deleteApplicationStatus,
    loadExternalDeployments,
    loadWebhookSources,
    loadWebhookSourceFilters,
    loadServers,
    refreshExternalDeployments,
    setIsLoading,
    setConnectionServers,
    setConfigDetails,
    setExternalDeployments,
    setServers,
    setExternalDeploymentsCount,
    setMaxAge,
    setSseStreamChannel,
    setLiveUpdate,
    setWebhookSources,
    setWebhookSourceFilters,
    addServerToServers,
    subscribeToSseStream,
    unsubscribeFromSseStream,
    updateApplicationStatus,
    updateWebhookSourceFilters,
    deleteWebhookSource,
    removeWebhookSource,
    setFilterEventSource,
    loadFilterEventSource,
    saveEventSource,
    storeFilters,
    getFilters,
    setPage,
    setCondition,
    setConnectionErrors,
} = folderExternalDeployments.actions;
export const toaster = ToastrFactory();
const url = 'api/v1';

export function* executeLoadExternalDeploymentsWithRefreshAction() {
    yield put(setMaxAge(MAX_AGE_NO_CACHE));
    yield call(executeLoadExternalDeploymentsAction);
    yield put(setMaxAge(DEFAULT_MAX_AGE));
}

export function* executeUpdateApplicationPackageStatusAction(action: PayloadAction<EndpointExternalDeploymentEvent<UpdateStatusEvent>>) {
    const { connectionServers, externalDeployments }: FolderExternalDeploymentsState = yield select(getFolderExternalDeployments);
    const { endpointId, state: event } = action.payload;
    let found = false;
    const updatedDeployments: Array<ExternalDeployment> = externalDeployments.map((deployment): ExternalDeployment => {
        const server = connectionServers.get(deployment.endpointId);
        if (deployment.applicationUid === event.applicationUid && event.destination === deployment.state.destination && server?.id === endpointId) {
            found = true;
            return {
                ...deployment,
                ...event,
            };
        }
        return deployment;
    });
    if (found) {
        yield put(setExternalDeployments(updatedDeployments));
    } else {
        yield call(executeLoadExternalDeploymentsWithRefreshAction);
    }
}

export function* executeDeleteApplicationPackageStatusAction(action: PayloadAction<EndpointExternalDeploymentEvent<DeleteStatusEvent>>) {
    const { connectionServers, externalDeployments }: FolderExternalDeploymentsState = yield select(getFolderExternalDeployments);
    const deleteEvent: DeleteStatusEvent = action.payload.state;
    const eventEndpointId = action.payload.endpointId;
    const updatedDeployments = externalDeployments.filter(({ applicationUid, endpointId, state }) => {
        const server = connectionServers.get(endpointId);
        return applicationUid !== deleteEvent.applicationUid || server?.id !== eventEndpointId || state.destination !== deleteEvent.destination;
    });
    yield put(setExternalDeployments(updatedDeployments));
}

export function* withLoadingState<R>(effect: CallEffect) {
    try {
        yield put(setIsLoading(true));
        const result: R = yield effect;
        return result;
    } finally {
        yield put(setIsLoading(false));
    }
}

export function* executeLoadExternalDeploymentsAction() {
    yield call(withFlagChangingState, call(executeLoadExternalDeployments), setIsLoading);
}

export function* executeLoadExternalDeployments() {
    const { page, maxAge, condition }: FolderExternalDeploymentsState = yield select(getFolderExternalDeployments);
    if (!page || !page.folderId) {
        return;
    }
    try {
        const {
            data: { count, connectionServers, externalDeployments, errors },
        } = yield call(
            httpGET,
            `/external-deployment?max-age=${maxAge}&folderId=${page.folderId}&page=${page.page}&resultsPerPage=${page.resultsPerPage}&orderBy=${mapOrderByValue(
                page.orderBy,
            )}&order=${page.order.toUpperCase()}&condition=${encodeURIComponent(condition)}`,
            true,
        );
        yield put(setConnectionServers(new Map(Object.entries(connectionServers))));
        yield put(setConnectionErrors(errors));
        yield put(setExternalDeployments(externalDeployments));
        yield put(setExternalDeploymentsCount(count));
    } catch (e: unknown) {
        const err = e as AxiosError;
        const errServerData: CustomConfiguration = err.response?.data;
        const errorMessage =
            'Error fetching status data. Check connection to ' + errServerData.title + ' HTTP Connection or check application logs for more details.';
        yield call(toaster.error, errorMessage);
    }
}

export function* executeSseStreamSubscribeAction() {
    const existingChannel: EventChannel<Event> = yield select(getApplicationStatusChannelSelector);
    if (existingChannel) {
        yield call(existingChannel.close);
    }
    const eventSrc: EventSource = yield call(registerEventSource, SSE_REGISTRATION_URL);
    const channel: EventChannel<Event> = yield call(subscribeToSse, eventSrc);
    yield put(setSseStreamChannel(channel));

    while (true) {
        const event: MessageEvent = yield take(channel);
        const data = angular.fromJson(event.data);
        switch (event.type) {
            case EVENT_TYPE.applicationPackageCreated:
                // might be needed in the future
                break;
            case EVENT_TYPE.applicationPackageDeleted:
                // might be needed in the future
                break;
            case EVENT_TYPE.applicationCreated:
                yield put(createApplicationStatus(data));
                break;
            case EVENT_TYPE.applicationDeleted:
                yield put(deleteApplicationStatus(data));
                break;
            case EVENT_TYPE.applicationChanged:
                yield put(updateApplicationStatus(data));
                break;
            default:
            //do nothing
        }
    }
}

export function* executeSseStreamUnsubscribeAction() {
    const channel: EventChannel<Event> = yield select(getApplicationStatusChannelSelector);
    if (channel) {
        yield call(channel.close);
    }
    yield put(setSseStreamChannel(undefined));
}

export function* executeCreateServerAction(action: PayloadAction<Server>) {
    const server = action.payload;
    const { data }: IHttpResponse<Server> = yield call(withLoadingState, call(httpPOST, `${url}/config`, server));
    yield call(toaster.success, `Server created successfully`);
    yield put(addServerToServers(data));
}

export function* executeSetupLiveUpdateAction(action: PayloadAction<SetupLiveUpdateRequest>) {
    const folderId = action.payload.folderId;
    const serverId = action.payload.serverId;
    const { data }: IHttpResponse<WebhookSourceAutoConfigDetails> = yield call(
        withLoadingState,
        call(httpPOST, `/external-deployment/setup/${serverId}?folderId=${folderId}`),
    );
    yield put(setConfigDetails(data));
    yield call(getWebhookSources, folderId);
}

export function* getWebhookSources(folderId: string) {
    const { data }: IHttpResponse<Array<WebhookSource>> = yield call(
        withLoadingState,
        call(httpGET, `${url}/config/byTypeAndTitle?configurationType=events.StatusWebhookEventSource&folderId=${folderId}&folderOnly=true`),
    );
    yield put(setWebhookSources(data));
}

export function* executeLoadWebhookSourceFiltersAction(action: PayloadAction<string>) {
    const webhookSourceId = action.payload;
    const { data: webhookSourceFilters }: IHttpResponse<WebhookSourceFilter> = yield call(
        withLoadingState,
        call(httpGET, `/external-deployment/filters/${webhookSourceId}`),
    );
    yield put(setWebhookSourceFilters(webhookSourceFilters));
}

export function* executeUpdateWebhookSourceFiltersAction(action: PayloadAction<SaveWebhookSourceFiltersRequest>) {
    const saveWebhookSourceFiltersRequest = action.payload;
    const { data: updatedSource }: IHttpResponse<WebhookSource> = yield call(
        withLoadingState,
        call(httpPUT, `/external-deployment/filters/${saveWebhookSourceFiltersRequest.webhookSourceId}`, saveWebhookSourceFiltersRequest.filteredFolders),
    );
    const webhookSources: Array<WebhookSource> = yield select(getWebhookSourcesSelector);
    const updatedWebhookSources: Array<WebhookSource> = webhookSources.map((webhookSource): WebhookSource => {
        if (webhookSource.id === updatedSource.id) return updatedSource;
        return webhookSource;
    });
    yield put(setWebhookSources(updatedWebhookSources));
}

export function* executeLoadServersAction(action: PayloadAction<string>) {
    const folderId: string = action.payload;
    const { data: servers }: IHttpResponse<Array<Server>> = yield call(
        withLoadingState,
        call(httpGET, `${url}/config/byTypeAndTitle?configurationType=configuration.StatusHttpConnection&folderId=${folderId}`),
    );
    yield put(setServers(servers));
}

export function* executeLoadWebhookSourcesAction(action: PayloadAction<string>) {
    const folderId = action.payload;
    yield call(getWebhookSources, folderId);
}

export function* executeDeleteWebhookSourceAction(action: PayloadAction<string>) {
    const webhookSourceId = action.payload;
    yield call(withLoadingState, call(httpDELETE, `${url}/config/${webhookSourceId}`));
    yield put(removeWebhookSource(webhookSourceId));
}

export function* executeLoadFilterEventSourceAction(action: PayloadAction<string>) {
    const eventSourceId = action.payload;
    const { data: eventSource }: IHttpResponse<WebhookEndpoint> = yield call(withLoadingState, call(httpGET, `${url}/config/${eventSourceId}`));
    yield put(setFilterEventSource(eventSource));
}

export function* executeSaveEventSourceAction(action: PayloadAction<WebhookEndpoint>) {
    const webhookEventSource = action.payload;
    yield call(withLoadingState, call(httpPUT, `${url}/config/${webhookEventSource.id}`, webhookEventSource));
    yield call(toaster.success, `Webhook endpoint updated successfully`);
}

export function* storeFiltersAction() {
    const { page, condition } = yield select(getFolderExternalDeployments);
    const clientSettings: ClientSettings = yield call(getAngularService, 'ClientSettings');
    clientSettings.setExternalDeploymentsFilters({ ...page, condition });
    const FiltersQueryParams: FilterQueryParams = yield call(getAngularService, 'FiltersQueryParams');
    FiltersQueryParams.update({ ...page, condition });
}

export function* getFiltersAction(action: PayloadAction<string>) {
    const folderId: string = action.payload;
    const clientSettings: ClientSettings = yield call(getAngularService, 'ClientSettings');
    const filters: ExternalDeploymentFilters = clientSettings.getExternalDeploymentsFilters();
    if (filters) {
        yield put(setCondition(filters.condition));
        yield put(
            setPage({
                folderId,
                page: filters.page,
                order: filters.order,
                orderBy: filters.orderBy,
                resultsPerPage: filters.resultsPerPage,
            }),
        );
    }
}

export default function* externalDeploymentsRootSaga() {
    yield all([
        takeEvery(loadExternalDeployments, executeLoadExternalDeploymentsAction),
        takeEvery(refreshExternalDeployments, executeLoadExternalDeploymentsWithRefreshAction),
        takeEvery(createApplicationStatus, executeLoadExternalDeploymentsWithRefreshAction),
        takeEvery(updateApplicationStatus, executeUpdateApplicationPackageStatusAction),
        takeEvery(deleteApplicationStatus, executeDeleteApplicationPackageStatusAction),
        takeEvery(subscribeToSseStream, executeSseStreamSubscribeAction),
        takeEvery(unsubscribeFromSseStream, executeSseStreamUnsubscribeAction),
        takeEvery(createServer, executeCreateServerAction),
        takeEvery(setLiveUpdate, executeSetupLiveUpdateAction),
        takeEvery(loadWebhookSourceFilters, executeLoadWebhookSourceFiltersAction),
        takeEvery(updateWebhookSourceFilters, executeUpdateWebhookSourceFiltersAction),
        takeEvery(loadServers, executeLoadServersAction),
        takeEvery(loadWebhookSources, executeLoadWebhookSourcesAction),
        takeEvery(deleteWebhookSource, executeDeleteWebhookSourceAction),
        takeEvery(loadFilterEventSource, executeLoadFilterEventSourceAction),
        takeEvery(saveEventSource, executeSaveEventSourceAction),
        takeEvery(storeFilters, storeFiltersAction),
        takeEvery(getFilters, getFiltersAction),
    ]);
}

//todo create action
export const checkStatus = (ci: { id: null; properties: Server; title: string; type: string }) =>
    httpPOST(`configurations/status`, ci).then((resp) => resp.data);

==> xlr-status-event-module-23.1.0-1005.113.jar: web/include/external-deployments/ducks/external-deployments.saga.ts <==
import { all, call, CallEffect, put, select, take, takeEvery } from 'redux-saga/effects';
import angular, { IHttpResponse } from 'angular';
import { AxiosError } from 'axios';
import { PayloadAction } from '@reduxjs/toolkit';
import { EventChannel } from 'redux-saga';
import { folderExternalDeployments, FolderExternalDeploymentsState } from './external-deployments.reducer';
import { Folder } from '../../../../../../../../../core/xlr-ui/app/types';
import {
    DeleteStatusEvent,
    EndpointExternalDeploymentEvent,
    ExternalDeploymentFilters,
    WebhookEndpoint,
    ExternalDeployment,
    SaveWebhookSourceFiltersRequest,
    Server,
    SetupLiveUpdateRequest,
    UpdateStatusEvent,
    WebhookSource,
    WebhookSourceAutoConfigDetails,
    WebhookSourceFilter,
} from '../external-deployment.types';
import { CustomConfiguration } from '../../../../../../../../../core/xlr-ui/app/types';
import { mapOrderByValue, registerEventSource, subscribeToSse } from '../helper/utils';
import { DEFAULT_MAX_AGE, EVENT_TYPE, MAX_AGE_NO_CACHE, SSE_REGISTRATION_URL } from '../constants';
import { getApplicationStatusChannelSelector, getFolderExternalDeployments, getWebhookSourcesSelector } from './external-deployments.selectors';
import ToastrFactory from '../../../../../../../../../core/xlr-ui/app/js/util/toastrFactory';
import { httpGET, httpPOST, httpPUT, httpDELETE } from '../../../../../../../../../core/xlr-ui/app/features/common/services/http';
import { withFlagChangingState } from '../../../../../../../../../core/xlr-ui/app/react/utils/saga-utils';
import getAngularService from '../../../../../../../../../core/xlr-ui/app/features/common/services/angular-accessor';
import { ClientSettings, FilterQueryParams } from '../../../../../../../../../core/xlr-ui/app/types';
import { BreadcrumbService } from '../../../../../../../../../core/xlr-ui/app/features/tasks/types/angular';

const {
    createApplicationStatus,
    createServer,
    deleteApplicationStatus,
    loadExternalDeployments,
    loadWebhookSources,
    loadWebhookSourceFilters,
    loadServers,
    refreshExternalDeployments,
    setIsLoading,
    setConnectionServers,
    setConfigDetails,
    setExternalDeployments,
    setServers,
    setExternalDeploymentsCount,
    setMaxAge,
    setSseStreamChannel,
    setLiveUpdate,
    setWebhookSources,
    setWebhookSourceFilters,
    addServerToServers,
    subscribeToSseStream,
    unsubscribeFromSseStream,
    updateApplicationStatus,
    updateBreadcrumbs,
    updateWebhookSourceFilters,
    deleteWebhookSource,
    removeWebhookSource,
    setFilterEventSource,
    loadFilterEventSource,
    saveEventSource,
    storeFilters,
    getFilters,
    setPage,
    setCondition,
    setConnectionErrors,
    setDisconnectedServers,
    patchExternalDeployments,
} = folderExternalDeployments.actions;
export const toaster = ToastrFactory();
const url = 'api/v1';

export function* executeLoadExternalDeploymentsWithRefreshAction() {
    yield put(setMaxAge(MAX_AGE_NO_CACHE));
    yield call(executeLoadExternalDeploymentsAction);
    yield put(setMaxAge(DEFAULT_MAX_AGE));
}

export function* executeUpdateApplicationPackageStatusAction(action: PayloadAction<EndpointExternalDeploymentEvent<UpdateStatusEvent>>) {
    const { connectionServers, externalDeployments }: FolderExternalDeploymentsState = yield select(getFolderExternalDeployments);
    const { endpointId, state: event } = action.payload;
    let found = false;
    const updatedDeployments: Array<ExternalDeployment> = externalDeployments.map((deployment): ExternalDeployment => {
        const server = connectionServers.get(deployment.endpointId);
        if (deployment.applicationUid === event.applicationUid && event.destinationUid === deployment.state.destinationUid && server?.id === endpointId) {
            found = true;
            return {
                ...deployment,
                ...event,
            };
        }
        return deployment;
    });
    if (found) {
        yield put(setExternalDeployments(updatedDeployments));
    } else {
        yield call(executeLoadExternalDeploymentsWithRefreshAction);
    }
}

export function* executeDeleteApplicationPackageStatusAction(action: PayloadAction<EndpointExternalDeploymentEvent<DeleteStatusEvent>>) {
    const { connectionServers, externalDeployments }: FolderExternalDeploymentsState = yield select(getFolderExternalDeployments);
    const deleteEvent: DeleteStatusEvent = action.payload.state;
    const eventEndpointId = action.payload.endpointId;
    const updatedDeployments = externalDeployments.filter(({ applicationUid, endpointId, state }) => {
        const server = connectionServers.get(endpointId);
        return applicationUid !== deleteEvent.applicationUid || server?.id !== eventEndpointId || state.destinationUid !== deleteEvent.destinationUid;
    });
    yield put(setExternalDeployments(updatedDeployments));
}

export function* withLoadingState<R>(effect: CallEffect) {
    try {
        yield put(setIsLoading(true));
        const result: R = yield effect;
        return result;
    } finally {
        yield put(setIsLoading(false));
    }
}

export function* executeLoadExternalDeploymentsAction() {
    yield call(withFlagChangingState, call(executeLoadExternalDeployments), setIsLoading);
}

export function* executeLoadExternalDeployments() {
    const { page, maxAge, condition }: FolderExternalDeploymentsState = yield select(getFolderExternalDeployments);
    if (!page || !page.folderId) {
        return;
    }
    try {
        const {
            data: { count, connectionServers, disconnectedServers, externalDeployments, errors },
        } = yield call(
            httpGET,
            `/external-deployment?max-age=${maxAge}&folderId=${page.folderId}&page=${page.page}&resultsPerPage=${page.resultsPerPage}&orderBy=${mapOrderByValue(
                page.orderBy,
            )}&order=${page.order.toUpperCase()}&condition=${encodeURIComponent(condition)}`,
            true,
        );
        yield put(setConnectionServers(new Map(Object.entries(connectionServers))));
        yield put(setConnectionErrors(errors));
        yield put(setExternalDeployments(externalDeployments));
        yield put(setExternalDeploymentsCount(count));
        yield put(setDisconnectedServers(disconnectedServers));
    } catch (e: unknown) {
        const err = e as AxiosError;
        const errServerData: CustomConfiguration = err.response?.data;
        const errorMessage =
            'Error fetching status data. Check connection to ' + errServerData.title + ' HTTP Connection or check application logs for more details.';
        yield call(toaster.error, errorMessage);
    }
}

export function* executeSseStreamSubscribeAction() {
    const existingChannel: EventChannel<Event> = yield select(getApplicationStatusChannelSelector);
    if (existingChannel) {
        yield call(existingChannel.close);
    }
    const eventSrc: EventSource = yield call(registerEventSource, SSE_REGISTRATION_URL);
    const channel: EventChannel<Event> = yield call(subscribeToSse, eventSrc);
    yield put(setSseStreamChannel(channel));

    while (true) {
        const event: MessageEvent = yield take(channel);
        const data = angular.fromJson(event.data);
        switch (event.type) {
            case EVENT_TYPE.applicationPackageCreated:
                // might be needed in the future
                break;
            case EVENT_TYPE.applicationPackageDeleted:
                // might be needed in the future
                break;
            case EVENT_TYPE.applicationCreated:
                yield put(createApplicationStatus(data));
                break;
            case EVENT_TYPE.applicationDeleted:
                yield put(deleteApplicationStatus(data));
                break;
            case EVENT_TYPE.applicationChanged:
                yield put(updateApplicationStatus(data));
                break;
            default:
            //do nothing
        }
    }
}

export function* executeSseStreamUnsubscribeAction() {
    const channel: EventChannel<Event> = yield select(getApplicationStatusChannelSelector);
    if (channel) {
        yield call(channel.close);
    }
    yield put(setSseStreamChannel(undefined));
}

export function* executeCreateServerAction(action: PayloadAction<Server>) {
    const server = action.payload;
    const { data }: IHttpResponse<Server> = yield call(withLoadingState, call(httpPOST, `${url}/config`, server));
    yield call(toaster.success, `Server created successfully`);
    yield put(addServerToServers(data));
}

export function* executeSetupLiveUpdateAction(action: PayloadAction<SetupLiveUpdateRequest>) {
    const folderId = action.payload.folderId;
    const serverId = action.payload.serverId;
    const { data }: IHttpResponse<WebhookSourceAutoConfigDetails> = yield call(
        withLoadingState,
        call(httpPOST, `/external-deployment/setup/${serverId}?folderId=${folderId}`),
    );
    yield put(setConfigDetails(data));
    yield call(getWebhookSources, folderId);
}

export function* getWebhookSources(folderId: string) {
    const { data }: IHttpResponse<Array<WebhookSource>> = yield call(
        withLoadingState,
        call(httpGET, `${url}/config/byTypeAndTitle?configurationType=events.StatusWebhookEventSource&folderId=${folderId}&folderOnly=true`),
    );
    yield put(setWebhookSources(data));
}

export function* executeLoadWebhookSourceFiltersAction(action: PayloadAction<string>) {
    const webhookSourceId = action.payload;
    try {
        const { data: webhookSourceFilters }: IHttpResponse<WebhookSourceFilter> = yield call(
            withLoadingState,
            call(httpGET, `/external-deployment/filters/${webhookSourceId}`),
        );
        yield put(setWebhookSourceFilters(webhookSourceFilters));
    } catch (e) {
        const err = e as AxiosError;
        yield call(toaster.error, err.message);
    }
}

export function* executeUpdateWebhookSourceFiltersAction(action: PayloadAction<SaveWebhookSourceFiltersRequest>) {
    const saveWebhookSourceFiltersRequest = action.payload;
    const { data: updatedSource }: IHttpResponse<WebhookSource> = yield call(
        withLoadingState,
        call(httpPUT, `/external-deployment/filters/${saveWebhookSourceFiltersRequest.webhookSourceId}`, saveWebhookSourceFiltersRequest.filteredFolders),
    );
    const webhookSources: Array<WebhookSource> = yield select(getWebhookSourcesSelector);
    const updatedWebhookSources: Array<WebhookSource> = webhookSources.map((webhookSource): WebhookSource => {
        if (webhookSource.id === updatedSource.id) return updatedSource;
        return webhookSource;
    });
    yield put(setWebhookSources(updatedWebhookSources));
}

export function* executeLoadServersAction(action: PayloadAction<string>) {
    const folderId: string = action.payload;
    const { data: servers }: IHttpResponse<Array<Server>> = yield call(
        withLoadingState,
        call(httpGET, `${url}/config/byTypeAndTitle?configurationType=configuration.StatusHttpConnection&folderId=${folderId}`),
    );
    yield put(setServers(servers));
}

export function* executeLoadWebhookSourcesAction(action: PayloadAction<string>) {
    const folderId = action.payload;
    yield call(getWebhookSources, folderId);
}

export function* executeDeleteWebhookSourceAction(action: PayloadAction<string>) {
    const webhookSourceId = action.payload;
    yield call(withLoadingState, call(httpDELETE, `external-deployment/delete/${webhookSourceId}`));
    yield put(removeWebhookSource(webhookSourceId));
}

export function* executeLoadFilterEventSourceAction(action: PayloadAction<string>) {
    const eventSourceId = action.payload;
    const { data: eventSource }: IHttpResponse<WebhookEndpoint> = yield call(withLoadingState, call(httpGET, `${url}/config/${eventSourceId}`));
    yield put(setFilterEventSource(eventSource));
}

export function* executeSaveEventSourceAction(action: PayloadAction<WebhookEndpoint>) {
    const webhookEventSource = action.payload;
    yield call(withLoadingState, call(httpPUT, `${url}/config/${webhookEventSource.id}`, webhookEventSource));
    yield call(toaster.success, `Webhook endpoint updated successfully`);
}

export function* storeFiltersAction() {
    const { page, condition } = yield select(getFolderExternalDeployments);
    const clientSettings: ClientSettings = yield call(getAngularService, 'ClientSettings');
    clientSettings.setExternalDeploymentsFilters({ ...page, condition });
    const FiltersQueryParams: FilterQueryParams = yield call(getAngularService, 'FiltersQueryParams');
    FiltersQueryParams.update({ ...page, condition });
}

export function* getFiltersAction(action: PayloadAction<string>) {
    const folderId: string = action.payload;
    const clientSettings: ClientSettings = yield call(getAngularService, 'ClientSettings');
    const filters: ExternalDeploymentFilters = clientSettings.getExternalDeploymentsFilters();
    if (filters) {
        yield put(setCondition(filters.condition));
        yield put(
            setPage({
                folderId,
                page: filters.page,
                order: filters.order,
                orderBy: filters.orderBy,
                resultsPerPage: filters.resultsPerPage,
            }),
        );
    }
}

export function* updateBreadcrumbsAction(action: PayloadAction<Folder>) {
    const folder = action.payload;

    const breadcrumbs = [
        {
            pathSuffix: 'folders/' + folder.id,
            label: folder.title,
            folder,
        },
        {
            pathSuffix: 'folders/' + folder.id + '/external-deployments',
            label: 'Application deployments',
            folder,
        },
        {
            label: 'Application Sources',
        },
    ];

    const breadcrumbService: BreadcrumbService = yield call(getAngularService, 'BreadcrumbsService');
    breadcrumbService.updateBreadcrumbs(breadcrumbs, false);
}

export function* executePatchExternalDeploymentsAction(action: PayloadAction<string>) {
    const webhookSourceId = action.payload;
    yield call(withLoadingState, call(httpPOST, `/external-deployment/patch/${webhookSourceId}`));
    yield call(toaster.success, `External deployments patched successfully`);
}

export default function* externalDeploymentsRootSaga() {
    yield all([
        takeEvery(loadExternalDeployments, executeLoadExternalDeploymentsAction),
        takeEvery(refreshExternalDeployments, executeLoadExternalDeploymentsWithRefreshAction),
        takeEvery(createApplicationStatus, executeLoadExternalDeploymentsWithRefreshAction),
        takeEvery(updateApplicationStatus, executeUpdateApplicationPackageStatusAction),
        takeEvery(deleteApplicationStatus, executeDeleteApplicationPackageStatusAction),
        takeEvery(subscribeToSseStream, executeSseStreamSubscribeAction),
        takeEvery(unsubscribeFromSseStream, executeSseStreamUnsubscribeAction),
        takeEvery(createServer, executeCreateServerAction),
        takeEvery(setLiveUpdate, executeSetupLiveUpdateAction),
        takeEvery(loadWebhookSourceFilters, executeLoadWebhookSourceFiltersAction),
        takeEvery(updateWebhookSourceFilters, executeUpdateWebhookSourceFiltersAction),
        takeEvery(loadServers, executeLoadServersAction),
        takeEvery(loadWebhookSources, executeLoadWebhookSourcesAction),
        takeEvery(deleteWebhookSource, executeDeleteWebhookSourceAction),
        takeEvery(loadFilterEventSource, executeLoadFilterEventSourceAction),
        takeEvery(saveEventSource, executeSaveEventSourceAction),
        takeEvery(storeFilters, storeFiltersAction),
        takeEvery(getFilters, getFiltersAction),
        takeEvery(updateBreadcrumbs, updateBreadcrumbsAction),
        takeEvery(patchExternalDeployments, executePatchExternalDeploymentsAction),
    ]);
}

//todo create action
export const checkStatus = (ci: { id: null; properties: Server; title: string; type: string }) =>
    httpPOST(`configurations/status`, ci).then((resp) => resp.data);
